/**
   class Path

   Copyright (C) 2002-4 Sebastian Pado
   
   This file is part of the DependencyVectors v2 distribution.
   
   DependencyVectors is free software; you can redistribute it
   and/or modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   DependencyVectors is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied
   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with DependencyVectors; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307 USA

**/

package de.ims.cl.dv.datastructures;

import java.util.*;

import de.ims.cl.dv.parsers.Encoding;

/** A Path is a list of Edges. **/

public class Path extends ArrayList<Edge> {
    
    private static final long serialVersionUID = 1;
    
    /** Add an Edge to a Path **/
    public void addEdge(Edge e) { /*  Needs real Edge */
	this.add(e);
    }
        
    /** returns the set of all nodes which are touched by the
        Path. Note that this is a set, so there won't be any
	duplicates. 
    **/
    
    public Set<Node> yield() {
    	int i;
    	Set<Node> allWords = new HashSet<Node>();
    	for (i=0;i<this.size();i++) {
    	    Edge currentEdge = (Edge) this.get(i);
    	    allWords.add(currentEdge.getFromNode());
    	    allWords.add(currentEdge.getToNode());
    	}
    	return allWords;
     }
    
    /** Returns the set of all relations on the Path. Note that this is a set,
	so there won't be any duplicates. 
    **/
    public HashSet<String> relSet() {
	int i;
	HashSet<String> allRels = new HashSet<String>();
	for (i=0;i<this.size();i++) {
	    Edge currentEdge = (Edge) this.get(i);
	    allRels.add(currentEdge.getRelation());
	}
	return allRels;
    }
    
    /** Returns a string representation of a Path: Edge1=Edge2=...
     **/
    public String toString() {
  	String acc = "";
  	ListIterator<Edge> li = this.listIterator();
  	while (li.hasNext()) {
  	    acc += li.next() + "=";
  	}
  	return acc.substring(0,acc.length()-1);
    }
    
    /** Constructs a Path from a string representation of a path
     **/
    public Path(String s, Encoding e) {
	super();
	String[] edges = s.split("=");
	for (int i=0; i<edges.length;i++) {
	    String[] tokens = edges[i].split(":");
	    if (tokens.length == 5) {
		String fromLemma = tokens[0];
		String fromPOS = tokens[1];
		String rel = tokens[2];
		String toPOS = tokens[3];
		String toLemma = tokens[4];
		this.add(new Edge(new Node(fromLemma,fromPOS,e),new Node(toLemma,toPOS,e),rel));
	    } else { 
		System.out.println("Error: "+tokens.length+" tokens in "+s);
	    }
	}
	if (this.isEmpty()) {
	    throw new NullPointerException(s);
	}

    }
    
    /** Initialises an empty path. **/
    
    public Path() {
	super();
    }
}
