/**
   class FreelingCorpus 

   This class has been created by Raquel Garrido Alhama.
   
   This file is an extension of the DependencyVectors v2 distribution.
   
   DependencyVectors is free software; you can redistribute it
   and/or modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   DependencyVectors is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied
   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with DependencyVectors; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307 USA

**/

package de.ims.cl.dv.parsers;

import java.io.*;
import java.util.*;
import java.util.regex.*;

import de.ims.cl.dv.datastructures.*;
import de.ims.cl.dv.util.*;


public class FreelingCorpus implements Corpus {    
	
	boolean isDebug = false;
	
    BufferedReader corpus;
    Logfile log;
    long    charcounter = 0; // count how much has been read from the corpus
    PosMapping   posmap = null;
    RelMapping   relmap = null;
    Pattern punctuation = Pattern.compile("^[^\\w]+$");
    
    Graph nextParse; // lookahead
    
    Map<String,String> relTrans = new HashMap<String,String>();
   
    
    private void increaseCounter(int inc) throws IOException { // increase charcounter and output
        long distance = 1000000;
        if ((charcounter/distance - (charcounter +=inc)/distance) < 0) { // printout every 10 mill lines
            log.println("Reading corpus byte "+(new Long(charcounter)).toString());
        } 
    }
    
    /**
       Returns the number of bytes read from the current corpus
    **/
    
    public long charcounter() {
        return charcounter;
    }
  
  

    /**
       Returns the next parse graph
    **/
    
    private Edge edgeTransform(Edge edge){
        @SuppressWarnings("unused")
		Node  to   = edge.getToNode();
        Node  from = edge.getFromNode();
        String rel = edge.getRelation();
        /*REVISAR si esto es necesario*/
        if (rel.equals("subj") && from.getPOS().equals("det")) {
            from.setPOS("n");
        }
        return edge;
    }
    

    public Graph fetchNext() throws IOException {
    
        Graph pg = new Graph(); 
        List<PreliminaryEdge> edges = new LinkedList<PreliminaryEdge>(); // store preliminary edges
       
        
		String line,lemma,lemma_pos,rel;
		lemma=lemma_pos=rel="";
        Integer nodeIndex = -1;
        Integer govIndex = 0;        
        Edge edge;
        Stack<Integer> governors = new Stack<Integer>();
        governors.push(govIndex); 
        int cov = 0; //verbal coordination counter
        
        String first_co_n_rel = "";
        

        line = corpus.readLine();
        
        //There are no more paragraphs left
        if (line == null || "".equals(line.trim()))
        	return null;
        //Read next paragraph (they are separated with an empty line)
        while (line != null && !line.trim().isEmpty()) {
            
            
            increaseCounter(line.length());
           
            
            line = line.trim();
            
            if (!line.equals("]")){
            	nodeIndex += 1; 
            	govIndex = governors.peek(); //peeking here we ensure that a new governor is not going to govern itself
            	if (isDebug)
            		System.err.println("the governor is going to be: "+govIndex);
            	
            	if(line.endsWith("[") ){ //the node governs at least another node 
            		
            		if ( nodeIndex >= 1){//first line is already in stack
            			governors.push(nodeIndex);
            			if (isDebug)
            				System.err.println("pushed in stack: "+nodeIndex);
            		}
            		//remove the [
            		line = line.substring(0,(line.length()-1));
            	}
            	
            	String[] tokens = line.split("/");
            	
            	//KNOWLEDGE: tokens[0] is the type of chunk
            	//			 tokens[1] is the grammar relationship with the node's governor
            	//			 tokens[2] is a tupple with the word, the lemma and the POS
            	
            	//Node
            	String[] tuple = tokens[2].split(" ");
            	lemma = tuple[1].trim();            	
            	lemma_pos = pos_transformation(tuple[2].trim().replace(")","").substring(0,1)); 
            	Node n = new Node(lemma, lemma_pos, nodeIndex, new FreelingEncoding());
            	pg.addNode(n);
            	if(isDebug){
	            	System.err.println("node info: ");
	            	System.err.println("lemma: "+lemma);
	            	System.err.println("lemma_pos: "+lemma_pos);
            	}
            	
            	
            	//PreliminaryEdge            	            	


            	rel = rel_transformation(tokens[1].trim());
            	
            	//Special treatment for coordination trees
            	
            	//---- verbal coordination
        		if (cov == 0){ //we only swap the first cov
        			cov = cov + swapverbcoordination(governors,nodeIndex, tokens[1].trim());
        			if (cov > 0)
        				rel = "punc";
        		}
        		//------------------------
        		
        		
        		//---- noun coordination
        		
        		if((first_co_n_rel != null) && (!"".equals(first_co_n_rel))){ //first co-n is changed in second iteration of subtree
        			rel = first_co_n_rel;
        			first_co_n_rel = "";
        		}
        		
        		
        		if ("coor-n".equals(tokens[0].trim())){ //chunk tag indicates a coordination subtree        			
        				governors.pop(); //pop the just added governor (coor-n ends with [)
        				governors.push(governors.peek()); //double the item to replace the popped (avoid empty when ] )        				
        				first_co_n_rel = rel;
        				rel = "punc";
        		}
        		//--------------------------
        		
        		
            	if (rel != null && !"".equals(rel) && !"top".equals(tokens[1].trim())){//avoid erroneus and top relations
            		if(isDebug)
            			System.err.println("adding edge: "+rel);
            		
            		//---- verbal coordination
            		if (cov > 0)  { //first co-v is swapped
            			Integer aux = nodeIndex;
            			nodeIndex = govIndex;
            			govIndex = aux;
            		}
            		//-----------------------------------------
            		
            		
            		
            		edges.add(new PreliminaryEdge(nodeIndex, govIndex, rel));
            		
            		//---- verbal coordination
            		if (cov > 0)   //re-swap to avoid losing a node
            			nodeIndex = govIndex;
            		//-------------------------------------------
            		
            		
            	}
            	if(isDebug){
	            	System.err.println("nodeIndex: "+nodeIndex);
	            	System.err.println("govIndex: "+govIndex);
	            	System.err.println("rel: "+rel);
            	}
            	
            } else{  //end of subtree
            	if(isDebug){
	            	System.err.println("now in stack: " + governors.size() + "elements");
	            	System.err.println("poping node "+governors.peek());
            	}
            	governors.pop();
            	
            }
                line = corpus.readLine();
        }  // end while
        
                
        // add edges to graph
        for (PreliminaryEdge pre: edges) {
            edge = pre.constructProperEdge(pg);
            edge = edgeTransform(edge);
            pg.addEdge(edge);
        }
        if (isDebug)
        	System.err.println(pg);
        return pg;
    }

    
    private int swapverbcoordination(Stack<Integer> governors, Integer nodeIndex, String freelingrel){
    	//If the node is the first co-v, we need to swap it with the top node
    	if ("co-v".equals(freelingrel)){
    		int size = governors.size();
    		
    		governors.clear();
    		for (int i = 0; i< size; i++){
    			governors.push(nodeIndex);
    		}
    		
    		
    		return 1;
    	
    	}
    	
    	return 0;
    }
    /** Read corpus from STDIN and write error output to <i>l</i>.  **/
    
       
    public FreelingCorpus(Logfile l, Map<String,String> mappings) throws IOException{
        
        this.log = l;
        this.corpus = new BufferedReader(new InputStreamReader(System.in));
               
        if (mappings.containsKey("posmapFile")) {
            String posmapFile = mappings.get("posmapFile");
            this.posmap = new PosMapping(posmapFile);
        }
        if (mappings.containsKey("relmapFile")) {
            String relmapFile = mappings.get("relmapFile");
            this.relmap = new RelMapping(relmapFile);
        }        
        nextParse = null;
        next();
    }


    public FreelingCorpus(String filename, Logfile l, Map<String,String> mappings) throws FileNotFoundException {
       
        this.corpus = new BufferedReader(new FileReader(filename));
        this.log = l;
        
        if (mappings.containsKey("posmapFile")) {
            String posmapFile = mappings.get("posmapFile");
            this.posmap = new PosMapping(posmapFile);
        }
        if (mappings.containsKey("relmapFile")) {
            String relmapFile = mappings.get("relmapFile");
            this.relmap = new RelMapping(relmapFile);
        }
        nextParse = null;
        next();
        
        
    }
    
    public FreelingCorpus(String filename, Logfile l, String posmapFile, String relmapFile) throws FileNotFoundException {
        
        this.corpus = new BufferedReader(new FileReader(filename));
        this.log = l;
        this.posmap = new PosMapping(posmapFile);
        this.relmap = new RelMapping(relmapFile);
        nextParse = null;
        next();
    }

    

    /**
     * Transforms the POS tag in Freeling to MINIPAR format
     * @param freeling_pos
     */
    
    private String pos_transformation(String freeling_pos){
    	return posmap.lookup(freeling_pos);
    }
    
    /**
     * Maps from freeling relation to MINIPAR relation
     * @param freeling_rel
     * @return
     */
    private String rel_transformation(String freeling_rel){
    	if (!"".equals(relmap.lookup(freeling_rel)))
    		return relmap.lookup(freeling_rel);    	
    	else
    		return "";
    	
    }
    
    static public Boolean charsInCommon(String str1, String str2){
        for (int i = 0; i < str1.length(); i++) {
            for (int j = 0; j < str2.length(); j++) {
                if (str1.charAt(i) == str2.charAt(j)) {
                    return true;
                }
            }
        }
        return false;
    }
    
    
    public String posLine2Sent(String line){
        String res = "";
        String[] words = line.split(" ");
        for (int i = 1; i < words.length; i++) {
            String word = words[i].split("\\|")[0];
            res = res + word + " ";
        }
        res = res.substring(0, res.length()-1);
        return res;
    }


	  /**
     * return next parse graph - or null, if there isn't a next one.
     */
    public Graph next() {
        try {                
        	 
                Graph currentParse = nextParse;
                nextParse = fetchNext();
                return currentParse;
                
        } catch (IOException e) {
        throw new RuntimeException("IO Exception while reading corpus...");
        }
    }
    
    public boolean hasNext() {
        return (nextParse != null);
    }



	public void remove() {		
		  throw new UnsupportedOperationException();    
	}
    
}
