/**
   class CommandLineMap

   Copyright (C) 2002-4 Sebastian Pado
   
   This file is part of the DependencyVectors v2 distribution.
   
   DependencyVectors is free software; you can redistribute it
   and/or modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   DependencyVectors is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied
   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE.  See the GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with DependencyVectors; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307 USA

**/

package de.ims.cl.dv.util;

import java.util.*;

/** CommandLineMap is provides a simple interface to command line parameters (unary) and options (nullary).
 **/

public class CommandLineMap {
    
    List<String> permittedKeys;
    
    private HashMap<String,String> arguments = new HashMap<String,String>();
    
    
    private boolean isakey(String s) {
	return ((s.startsWith("--")) &&
		(permittedKeys.contains(stripFirstChars(s))));
    }
    
    private String stripFirstChars(String s) {
	return s.substring(2);
    }

    /** tests whether a particular command line argument exists
     **/
    public boolean exists(String key) {
	return arguments.containsKey(key);
    }
    
    /** tests whether a particular command line argument is an option. Will
	throw a ClassCastException if the key does not exist.  
     **/
    public boolean isOption(String key) {
	return ((String) arguments.get(key)).equals("");
    }
    
    /** returns the value of a particular command line argument key. 
	Returns null if the key is an option. Will throw a ClassCastException if the key does not exist.  
     **/

    public String get(String key) {
	return ((String) arguments.get(key));
    }

    /** CommandLineMap is initialized with actual command line
     * arguments (args) and the set of known parameters options
     * (knownArgs) specified as an array of Strings.
     CommandLineMap assumes a "long GNU" style for parameters, i.e. "--key value" for parameters and 
     "--option" for options.
    **/

    public CommandLineMap(String[] args, List<String> knownArgs) {
	int i = 0;
	permittedKeys = knownArgs;
	while (i < args.length) {
	    if (isakey(args[i])) {
		if ((i==(args.length-1)) || (isakey(args[i+1]))) { // this is an option
		    arguments.put(stripFirstChars(args[i++]),"");
		}
		else {
		    arguments.put(stripFirstChars(args[i++]),args[i++]); // hope that the expression is evaluates left-to-right
		}
	    } else {
		System.err.println("Command Line Error: expected known key instead of "+args[i]+"!");
		System.exit(0);
	    }
	}
    }
}
